"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MemoryBufferWindow = void 0;
const memory_1 = require("langchain/memory");
const logWrapper_1 = require("../../../utils/logWrapper");
class MemoryChatBufferSingleton {
    constructor() {
        this.memoryBuffer = new Map();
    }
    static getInstance() {
        if (!MemoryChatBufferSingleton.instance) {
            MemoryChatBufferSingleton.instance = new MemoryChatBufferSingleton();
        }
        return MemoryChatBufferSingleton.instance;
    }
    async getMemory(sessionKey, memoryParams) {
        await this.cleanupStaleBuffers();
        let memoryInstance = this.memoryBuffer.get(sessionKey);
        if (memoryInstance) {
            memoryInstance.last_accessed = new Date();
        }
        else {
            const newMemory = new memory_1.BufferWindowMemory(memoryParams);
            memoryInstance = {
                buffer: newMemory,
                created: new Date(),
                last_accessed: new Date(),
            };
            this.memoryBuffer.set(sessionKey, memoryInstance);
        }
        return memoryInstance.buffer;
    }
    async cleanupStaleBuffers() {
        var _a;
        const oneHourAgo = new Date(Date.now() - 60 * 60 * 1000);
        for (const [key, memoryInstance] of this.memoryBuffer.entries()) {
            if (memoryInstance.last_accessed < oneHourAgo) {
                await ((_a = this.memoryBuffer.get(key)) === null || _a === void 0 ? void 0 : _a.buffer.clear());
                this.memoryBuffer.delete(key);
            }
        }
    }
}
class MemoryBufferWindow {
    constructor() {
        this.description = {
            displayName: 'Window Buffer Memory',
            name: 'memoryBufferWindow',
            icon: 'fa:database',
            group: ['transform'],
            version: 1,
            description: 'Stores the chat history in a windowed buffer. Refreshes on restart.',
            defaults: {
                name: 'Window Buffer Memory',
            },
            codex: {
                categories: ['AI'],
                subcategories: {
                    AI: ['Memory'],
                },
                resources: {
                    primaryDocumentation: [
                        {
                            url: 'https://docs.n8n.io/integrations/builtin/cluster-nodes/sub-nodes/n8n-nodes-langchain.memorybufferwindow/',
                        },
                    ],
                },
            },
            inputs: [],
            outputs: ["ai_memory"],
            outputNames: ['Memory'],
            properties: [
                {
                    displayName: 'Session Key',
                    name: 'sessionKey',
                    type: 'string',
                    default: 'chat_history',
                    description: 'The key to use to store the memory in the workflow data',
                },
                {
                    displayName: 'Context Window Length',
                    name: 'contextWindowLength',
                    type: 'number',
                    default: 5,
                    description: 'The number of previous messages to consider for context',
                },
            ],
        };
    }
    async supplyData(itemIndex) {
        const sessionKey = this.getNodeParameter('sessionKey', itemIndex);
        const contextWindowLength = this.getNodeParameter('contextWindowLength', itemIndex);
        const workflowId = this.getWorkflow().id;
        const memoryInstance = MemoryChatBufferSingleton.getInstance();
        const memory = await memoryInstance.getMemory(`${workflowId}__${sessionKey}`, {
            k: contextWindowLength,
            inputKey: 'input',
            memoryKey: 'chat_history',
            outputKey: 'output',
            returnMessages: true,
        });
        return {
            response: (0, logWrapper_1.logWrapper)(memory, this),
        };
    }
}
exports.MemoryBufferWindow = MemoryBufferWindow;
//# sourceMappingURL=MemoryBufferWindow.node.js.map